//
//  CLPageView.m
//  CLImClient_Example
//
//  Created by SC on 2021/9/18.
//  Copyright © 2021 253.com. All rights reserved.
//

#import "CLPageView.h"

@interface CLPageView ()

@property (strong, nonatomic) NSMutableArray <UILabel *>*viewArray;
@end

@implementation CLPageView

- (instancetype)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
    }
    return self;
}
- (instancetype)initWithFrame:(CGRect)frame
                     dotCount:(NSInteger)count
                      dotSize:(CGSize)size
                  selectColor:(UIColor *)selectColor
                unSelectColor:(UIColor *)color
{
    self = [super initWithFrame:frame];
    if (self) {
        self.dotCount = count;
        self.dotSize = size;
        self.selectColor = selectColor;
        self.unselectColor = color;
    }
    return self;
}
- (void)setDotCount:(NSInteger)dotCount
{
    _dotCount = dotCount;
    for (UILabel *label in self.viewArray) {
        [label removeFromSuperview];
    }
    [self.viewArray removeAllObjects];
    if (dotCount > 0) {
        CGFloat space = (self.frame.size.width - (self.dotSize.width * dotCount)) / dotCount;
        CGFloat y = (self.frame.size.height - self.dotSize.height) / 2;
        for (NSInteger i = 0; i < dotCount; i++) {
            UILabel *label = [UILabel new];
            label.textAlignment = NSTextAlignmentCenter;
            label.textColor = self.textColor;
            label.font = self.font;
            [self addSubview:label];
            [self.viewArray addObject:label];
            
            label.frame = CGRectMake(space / 2 + self.dotSize.width * i + space * i, y, self.dotSize.width, self.dotSize.height);
            label.layer.cornerRadius = self.dotSize.width / 2;
            label.clipsToBounds = YES;
            if (i < self.selectCount) {
                label.text = [NSString stringWithFormat:@"%ld", i + 1];
                label.backgroundColor = self.selectColor;
            } else {
                label.text = @"";
                label.backgroundColor = self.unselectColor;
            }
        }
    }
}
- (void)setDotSize:(CGSize)dotSize
{
    _dotSize = dotSize;
    for (NSInteger i = 0; i < self.viewArray.count; i++) {
        UILabel *label = self.viewArray[i];
        CGFloat space = (self.frame.size.width - (dotSize.width * self.viewArray.count)) / self.viewArray.count;
        CGFloat y = (self.frame.size.height - dotSize.height) / 2;
        label.frame = CGRectMake(space / 2 + dotSize.width * i + space * i, y, dotSize.width, dotSize.height);
        label.layer.cornerRadius = dotSize.width / 2;
    }
}
- (void)setSelectCount:(NSInteger)selectCount
{
    _selectCount = selectCount;
    for (NSInteger i = 0; i < self.viewArray.count; i++) {
        UILabel *label = self.viewArray[i];
        if (i < selectCount) {
            label.text = [NSString stringWithFormat:@"%ld", i + 1];
            label.backgroundColor = self.selectColor;
        } else {
            label.text = @"";
            label.backgroundColor = self.unselectColor;
        }
    }
}
- (void)setSelectColor:(UIColor *)selectColor
{
    _selectColor = selectColor;
    for (NSInteger i = 0; i < self.viewArray.count; i++) {
        UILabel *label = self.viewArray[i];
        if (i < self.selectCount) {
            label.backgroundColor = selectColor;
        } else {
            label.backgroundColor = self.unselectColor;
        }
    }
}
- (void)setUnselectColor:(UIColor *)unselectColor
{
    _unselectColor = unselectColor;
    for (NSInteger i = 0; i < self.viewArray.count; i++) {
        UILabel *label = self.viewArray[i];
        if (i < self.selectCount) {
            label.backgroundColor = self.selectColor;
        } else {
            label.backgroundColor = unselectColor;
        }
    }
}
- (void)setFont:(UIFont *)font
{
    _font = font;
    for (UILabel *label in self.viewArray) {
        label.font = font;
    }
}
- (void)setTextColor:(UIColor *)textColor
{
    _textColor = textColor;
    for (UILabel *label in self.viewArray) {
        label.textColor = textColor;
    }
}
- (NSMutableArray<UILabel *> *)viewArray
{
    if (!_viewArray) {
        _viewArray = [NSMutableArray arrayWithCapacity:0];
    }
    return _viewArray;
}

@end
