//
//  CLRealProcessView.m
//  CLRealPersonSDK
//
//  Created by ydong on 2021/9/18.
//  Copyright © 2021 ydong. All rights reserved.
//

#import "CLRealProcessView.h"

#define TimeInterval 0.015

#define lineWH self.frame.size.width

@interface CLRealProcessView()

@property (nonatomic,strong) CAShapeLayer *progressLayer;

@property (strong, nonatomic) NSTimer *timer;

@property (assign, nonatomic) CGFloat progress;

// 保留
@property (assign, nonatomic) CGFloat currentProgress;

@property (nonatomic, assign) CGFloat lineWidth;

@property (nonatomic, strong) UILabel *processLabel;

@end

@implementation CLRealProcessView

- (instancetype)initWithFrame:(CGRect)frame withLineWidth:(CGFloat)lineWidth {
    self.lineWidth = lineWidth;
    return [self initWithFrame:frame];
}

- (instancetype)initWithFrame:(CGRect)frame {
    if (self = [super initWithFrame:frame]) {
        [self createLayerPath:self.lineWidth];
        self.backgroundColor = UIColor.clearColor;
    }
    return self;
}

- (void)createLayerPath:(CGFloat)lineWidth {
    if (self.lineWidth<0.1) { self.lineWidth = 2*KWidthScale; }
    UIBezierPath *circlePath = [UIBezierPath bezierPathWithArcCenter:CGPointMake(lineWH/2, lineWH/2) radius:(lineWH-self.lineWidth)/2.0 startAngle:M_PI/2+0.025 endAngle:5*M_PI/2+0.025 clockwise:YES];
    
    CAShapeLayer *bgLayer = [CAShapeLayer layer];
    bgLayer.frame = self.bounds;
    bgLayer.fillColor = [UIColor clearColor].CGColor;
    bgLayer.lineWidth = self.lineWidth;
    bgLayer.strokeColor = HexColor(@"#B6D0FF").CGColor;
    bgLayer.strokeStart = 0;
    bgLayer.strokeEnd = 1;
    bgLayer.lineCap = kCALineCapRound;
    bgLayer.path = circlePath.CGPath;
    [self.layer addSublayer:bgLayer];
    
    _progressLayer = [CAShapeLayer layer];
    _progressLayer.frame = self.bounds;
    _progressLayer.fillColor = [UIColor clearColor].CGColor;
    _progressLayer.lineWidth = self.lineWidth;
    _progressLayer.lineCap = kCALineCapRound;
    _progressLayer.strokeColor = HexColor(@"#B6D0FF").CGColor;
    _progressLayer.strokeStart = 0;
    _progressLayer.strokeEnd = 0;
    _progressLayer.path = circlePath.CGPath;
    [self.layer addSublayer:_progressLayer];
    
    CAGradientLayer *gradientLayer = [CAGradientLayer layer];
    gradientLayer.frame = self.bounds;
    [self.layer addSublayer:gradientLayer];
    
    CAGradientLayer *leftGradientLayer = [CAGradientLayer layer];
    leftGradientLayer.frame = CGRectMake(0, 0, lineWH/2, lineWH);
    [leftGradientLayer setColors:[NSArray arrayWithObjects:(id)HexColor(@"#5454FF").CGColor, (id)HexColor(@"#2D77FF").CGColor, nil]];
    [leftGradientLayer setLocations:@[@0.0,@1.0]];
    [leftGradientLayer setStartPoint:CGPointMake(0, 0)];
    [leftGradientLayer setEndPoint:CGPointMake(0, 1)];
    [gradientLayer addSublayer:leftGradientLayer];
    
    CAGradientLayer *rightGradientLayer = [CAGradientLayer layer];
    rightGradientLayer.frame = CGRectMake(lineWH/2, 0, lineWH/2, lineWH);
    [rightGradientLayer setColors:[NSArray arrayWithObjects:(id)HexColor(@"#5454FF").CGColor, (id)HexColor(@"#7F2DFF").CGColor, nil]];
    [rightGradientLayer setLocations:@[@0.0,@1.0]];
    [rightGradientLayer setStartPoint:CGPointMake(0, 0)];
    [rightGradientLayer setEndPoint:CGPointMake(0, 1)];
    [gradientLayer addSublayer:rightGradientLayer];
    [gradientLayer setMask:_progressLayer];
}

- (void)configAnimate {
    [self.progressLayer removeAllAnimations];
    CABasicAnimation *animation = [CABasicAnimation animationWithKeyPath:@"strokeEnd"];
    animation.fromValue = @0;
    animation.toValue = [NSNumber numberWithDouble:self.progress];
    animation.duration = self.durationTime;
    animation.fillMode = kCAFillModeForwards;
    animation.removedOnCompletion = NO;
    [self.progressLayer addAnimation:animation forKey:@"strokeEnd-"];
}

- (void)stopLabelTime {
    [self deleteTimer];
}

- (void)resetAction {
    if (self.durationTime<1) {
        self.durationTime = 1;
    }
    [self deleteTimer];
    [self.progressLayer removeAllAnimations];
    self.progressLayer.strokeEnd = 0;
    self.progress = 1;
    self.currentProgress = 0;
    self.processLabel.text = @"0s";
    [self.progressLayer removeAllAnimations];
}

- (void)startTimer {
    [self deleteTimer];
    self.timer = [NSTimer scheduledTimerWithTimeInterval:TimeInterval target:self selector:@selector(animate:) userInfo:nil repeats:YES];
}

- (void)animate:(NSTimer *)time {
    if (self.currentProgress <= self.progress){
        self.currentProgress += TimeInterval*self.progress/self.durationTime;
        if (_processLabel) {
            self.processLabel.text = [NSString stringWithFormat:@"%ds",(int)(self.currentProgress*self.durationTime)];
        }
    }
    else{
        if (_processLabel) {
            self.processLabel.text = [NSString stringWithFormat:@"%ds",(int)self.durationTime];
        }
        [self deleteTimer];
        if (self.delegate && [self.delegate respondsToSelector:@selector(endProcess:)]) {
            [self.delegate endProcess:self];
        }
    }
}

- (void)deleteTimer {
    if (self.timer) {
        [self.timer invalidate];
        self.timer = nil;
    }
}

- (void)startAction {
    if (self.showProcessLabel) {
        [self addSubview:self.processLabel];
    }
    if (self.durationTime<1) {
        self.durationTime = 1;
    }
    [self deleteTimer];
    [self.progressLayer removeAllAnimations];
    self.progressLayer.strokeEnd = 0;
    self.progress = 1;
    self.currentProgress = 0;
    [self configAnimate];
    [self startTimer];
}

- (UILabel *)processLabel {
    if (!_processLabel) {
        _processLabel = [[UILabel alloc] initWithFrame:self.bounds];
        _processLabel.textAlignment = NSTextAlignmentCenter;
        _processLabel.textColor = HexColor(@"#333333");
        _processLabel.backgroundColor = UIColor.clearColor;
        _processLabel.font = [UIFont systemFontOfSize:14];
    }
    return _processLabel;
}

@end
