//
//  CLVideoViewController.m
//  CLRealPersonDemo
//
//  Created by ydong on 2021/9/16.
//  Copyright © 2021 ydong. All rights reserved.
//

#import "CLVideoViewController.h"
#import <CLRealPersonSDK/CLRealPersonSDK.h>
#import "CLRealProcessView.h"
#import "CLPageView.h"
#import "CLResToastView.h"
#import "CLSucessViewController.h"
#import <AVFoundation/AVAudioPlayer.h>
#import <FLAnimatedImage/FLAnimatedImage.h>

#define ActionDuration  2

@interface CLVideoViewController ()<CLTimeoutToastViewDelegate>

@property (nonatomic, strong) UIView *recordView;

@property (nonatomic, strong) UIImageView *picImgView;

@property (nonatomic, strong) UILabel *msgLabel;

@property (nonatomic, strong) CLRealPersonManager *manager;

@property (nonatomic, strong) CLRealProcessView *videoProcess;

@property (nonatomic, strong) AVAudioPlayer *player;

@property (nonatomic, assign) BOOL shouldPlay;

@property (nonatomic, strong) UIImageView *scanImgView;

@property (nonatomic, strong) FLAnimatedImageView *actImgView;

@property (nonatomic, strong) CLPageView *pageView;

@property (nonatomic, strong) CLRealProcessView *circleView;

@property (nonatomic, strong) UIButton *btn;

@property (nonatomic, strong) CLResToastView *toastView;

@property (nonatomic, assign) CGFloat lastBrightness;

@end

@implementation CLVideoViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
#warning 程序后台或者页面被覆盖应停止识别流程
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(enterBack) name:UIApplicationDidEnterBackgroundNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(becomeActive) name:UIApplicationWillEnterForegroundNotification object:nil];
    
    self.view.backgroundColor = HexColor(@"FFFFFF");
    self.navigationController.navigationBarHidden = NO;
    self.navigationController.navigationBar.tintColor = UIColor.blackColor;
    self.navigationController.navigationBar.translucent = NO;
    
    self.btn = [UIButton buttonWithType:UIButtonTypeCustom];
    [self.btn setImage:[UIImage imageNamed:@"muc_on"] forState:UIControlStateNormal];
    [self.view addSubview:self.btn];
    self.shouldPlay = YES;
    [self.btn addTarget:self action:@selector(mucBtnClick:) forControlEvents:UIControlEventTouchUpInside];
    [self.btn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(self.view.mas_top).offset(11*KHeightScale);
        make.left.mas_equalTo(self.view.mas_left).offset(15*KWidthScale);
        make.size.mas_equalTo(CGSizeMake(30, 31));
    }];
    
    self.videoProcess = [[CLRealProcessView alloc] initWithFrame:CGRectMake(SCREEN_WIDTH-46*KWidthScale, 18*KWidthScale, 28*KWidthScale , 28*KWidthScale) withLineWidth:ceil(1.5*KWidthScale)];
    [self.view addSubview:self.videoProcess];
    self.videoProcess.showProcessLabel = YES;
    self.videoProcess.durationTime = 30;
    [self.videoProcess startAction];
    
    self.msgLabel = [[UILabel alloc] init];
    self.msgLabel.text = @"请面向屏幕";
    self.msgLabel.font = [UIFont boldSystemFontOfSize:20];
    self.msgLabel.textAlignment = NSTextAlignmentCenter;
    self.msgLabel.backgroundColor = UIColor.whiteColor;
    self.msgLabel.textColor = HexColor(@"#333333");
    self.msgLabel.layer.cornerRadius = 2;
    [self.view addSubview:self.msgLabel];
    [self.msgLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(self.view.mas_top).offset(37*KHeightScale);
        make.centerX.mas_equalTo(self.view.mas_centerX);
        make.size.mas_equalTo(CGSizeMake(SCREEN_WIDTH-120*KWidthScale, 28));
    }];
    
    self.recordView = [[UIView alloc] initWithFrame:CGRectMake((SCREEN_WIDTH-226*KWidthScale)/2, 37*KHeightScale+28+45*KWidthScale , 226*KWidthScale, 226*KWidthScale)];
    [self.view addSubview:self.recordView];
    self.recordView.layer.cornerRadius = 226*KWidthScale/2.;
    self.recordView.layer.masksToBounds = YES;
    
    self.picImgView = [[UIImageView alloc] init];
    self.picImgView.image = [UIImage imageNamed:@"real_pic"];
    [self.recordView addSubview:self.picImgView];
    [self.picImgView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.mas_equalTo(self.recordView.mas_centerX);
        make.centerY.mas_equalTo(self.recordView.mas_centerY);
        make.size.mas_equalTo(CGSizeMake(119*KWidthScale, 158*KWidthScale));
    }];
    
    self.actImgView = [[FLAnimatedImageView alloc] init];
    [self.view addSubview:self.actImgView];
    [self.actImgView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(self.recordView.mas_bottom).offset(44*KHeightScale);
        make.centerX.mas_equalTo(self.recordView.mas_centerX);
        make.size.mas_equalTo(CGSizeMake(123*KWidthScale, 111*KWidthScale));
    }];
    
    self.pageView = [[CLPageView alloc] initWithFrame:CGRectMake((SCREEN_WIDTH-123*KWidthScale)/2., 37*KHeightScale+28+45*KWidthScale+45*KWidthScale+226*KWidthScale+145*KWidthScale, 123*KWidthScale, 20*KWidthScale) dotCount:3 dotSize:CGSizeMake(20*KWidthScale, 20*KWidthScale) selectColor:HexColor(@"#2D77FF") unSelectColor:HexColor(@"#DCE4EF")];
    [self.view addSubview:self.pageView];
    self.pageView.textColor = UIColor.whiteColor;
    
    self.circleView = [[CLRealProcessView alloc] initWithFrame:CGRectMake(self.recordView.frame.origin.x-12*KWidthScale, self.recordView.frame.origin.y-12*KWidthScale, self.recordView.frame.size.height + 24*KWidthScale, self.recordView.frame.size.height + 24*KWidthScale) withLineWidth:4*KWidthScale];
    [self.view addSubview:self.circleView];
    
    self.toastView = [[CLResToastView alloc] init];
    self.toastView.frame = CGRectMake(44*KWidthScale, SCREEN_HEIGHT/2.5, SCREEN_WIDTH-88*KWidthScale, 118*KHeightScale);
    self.toastView.delegate = self;
    
    self.scanImgView = [[UIImageView alloc] initWithFrame:CGRectMake(0, -75*KWidthScale, self.recordView.frame.size.width, 75*KWidthScale)];
    self.scanImgView.image = [UIImage imageNamed:@"real_scan"];
    [self.recordView addSubview:self.scanImgView];
    
    self.manager = [[CLRealPersonManager alloc] initWithRecordView:self.recordView];
    [self startLiveDetectWithActionsHandler];
    
    self.btn.hidden = YES;
    self.actImgView.hidden = YES;
    self.pageView.hidden = YES;
    self.shouldPlay = NO;
    // 初始化音频，加快播放速度
    [self playActionMusic:@"0"];
    self.navigationItem.leftBarButtonItem = [[UIBarButtonItem alloc] initWithImage:[UIImage imageNamed:@"ico_back"] style:(UIBarButtonItemStyleDone) target:self action:@selector(back)];
}

-(void)back {
    [self.navigationController popViewControllerAnimated:YES];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    self.lastBrightness = [UIScreen mainScreen].brightness;
    [UIScreen mainScreen].brightness = 1.0;
}

- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
    if (self.manager) {
        [self.manager stopLiveDetect];
    }
    [UIScreen mainScreen].brightness = self.lastBrightness;
}

- (void)reset {
    [self.manager stopLiveDetect];
    
    self.btn.hidden = YES;
    self.shouldPlay = NO;
    [self.btn setImage:[UIImage imageNamed:@"muc_on"] forState:UIControlStateNormal];
    self.actImgView.hidden = YES;
    self.pageView.hidden = YES;
    self.videoProcess.hidden = NO;
    [self.videoProcess startAction];
    [self.circleView resetAction];
    self.scanImgView.hidden = YES;
    [self.scanImgView.layer removeAllAnimations];
    self.msgLabel.text = @"请面向屏幕";
    self.picImgView.image = [UIImage imageNamed:@"real_pic"];
}

- (void)updateTitleWithDic:(NSDictionary *)params {
    if ([params[@"type"] isEqualToString:@"1"]) {
        if([params[@"result"] isEqualToString:@"1"]) {
            self.msgLabel.text = @"请保持姿势不变";
            self.picImgView.image = [UIImage imageNamed:@"real_pic_r"];
        }
        else if ([params[@"result"] isEqualToString:@"2"]) {
            self.msgLabel.text = @"请面向屏幕";
            self.picImgView.image = [UIImage imageNamed:@"real_pic"];
        }
        else if ([params[@"result"] isEqualToString:@"3"]) {
            self.msgLabel.text = @"请保持脸在检测框内";
            self.picImgView.image = [UIImage imageNamed:@"real_pic"];
        }
        else if ([params[@"result"] isEqualToString:@"4"]) {
            self.msgLabel.text = @"请靠近一点";
            self.picImgView.image = [UIImage imageNamed:@"real_pic"];
        }
        else if ([params[@"result"] isEqualToString:@"5"]) {
            self.msgLabel.text = @"当前光线过暗，请调整";
            self.picImgView.image = [UIImage imageNamed:@"real_pic"];
        }
        else if ([params[@"result"] isEqualToString:@"6"]) {
            self.msgLabel.text = @"当前光线模糊，请调整";
            self.picImgView.image = [UIImage imageNamed:@"real_pic"];
        }
        else if ([params[@"result"] isEqualToString:@"7"]) {
             self.msgLabel.text = @"请露出全脸";
            self.picImgView.image = [UIImage imageNamed:@"real_pic"];
        }
        else if ([params[@"result"] isEqualToString:@"7"]) {
            // self.msgLabel.text = @"其他错误，请继续";
            self.picImgView.image = [UIImage imageNamed:@"real_pic"];
        }
    }
    else if ([params[@"type"] isEqualToString:@"2"]) {
        if([params[@"result"] isEqualToString:@"-1"]) {
            // 开始相关动作
            self.btn.hidden = NO;
            self.shouldPlay = YES;
            self.actImgView.hidden = NO;
            self.pageView.dotCount = [params[@"number"] intValue];
            self.pageView.selectCount = 1;
            self.pageView.hidden = NO;
            self.circleView.durationTime = ActionDuration*[params[@"number"] intValue];
            [self.circleView startAction];
            return;
        }
        else if([params[@"result"] isEqualToString:@"0"]) {
            self.msgLabel.text = @"请眨眼";
        }
        else if ([params[@"result"] isEqualToString:@"4"]) {
            self.msgLabel.text = @"请抬头";
        }
        else if ([params[@"result"] isEqualToString:@"5"]) {
            self.msgLabel.text = @"请低头";
        }
        else if ([params[@"result"] isEqualToString:@"6"]) {
            self.msgLabel.text = @"请张嘴";
        }
        else if ([params[@"result"] isEqualToString:@"7"]) {
            self.msgLabel.text = @"请摇头";
        }
        else {
            NSLog(@"请处理未知动作");
            [self.manager stopLiveDetect];
            return;
        }
        self.pageView.selectCount = [params[@"index"] intValue]+1;
        NSString *path = [[NSBundle mainBundle] pathForResource:[NSString stringWithFormat:@"act_%@", params[@"result"]] ofType:@"gif"];
        if (path) {
            NSData *dataOfGif = [NSData dataWithContentsOfFile: path];
            FLAnimatedImage *image = [FLAnimatedImage animatedImageWithGIFData:dataOfGif];
            self.actImgView.animatedImage = image;
        }
        [self playActionMusic:[NSString stringWithFormat:@"muc_%@", params[@"result"]]];
    }
    else if ([params[@"type"] isEqualToString:@"3"]) {
        [UIScreen mainScreen].brightness = self.lastBrightness;
        if([params[@"result"] isEqualToString:@"1"]) {
            [self.videoProcess stopLabelTime];
            [self.circleView stopLabelTime];
            self.videoProcess.hidden = YES;
            self.btn.hidden = YES;
            self.pageView.hidden = YES;
            self.actImgView.hidden = YES;
            self.msgLabel.text = @"检测中";
            self.scanImgView.hidden = NO;
            [self.scanImgView.layer removeAllAnimations];
            CABasicAnimation *animation = [CABasicAnimation animationWithKeyPath:@"position"];
            animation.duration = 3;
            animation.repeatCount = HUGE_VALF;
            animation.fromValue = [NSValue valueWithCGPoint:self.scanImgView.layer.position];
            animation.toValue = [NSValue valueWithCGPoint:CGPointMake(self.scanImgView.layer.position.x, self.scanImgView.layer.position.y+75*KWidthScale+self.recordView.frame.size.height)];
            [self.scanImgView.layer addAnimation:animation forKey:@"move-layer"];
        }
    }
    else if ([params[@"type"] isEqualToString:@"4"]) {
        /// 不转发处理逻辑
        [self.manager resultWithDetectUrl:params[@"result"] isReTran:NO];
        
        /// 转发处理逻辑。如果需要转发url（RS_UPLOAD_URL）改为自己服务器地址、demo中只是参考
        /*
        NSLog(@"-- 处理转发 --");
        NSMutableURLRequest *request = [[AFHTTPRequestSerializer serializer] multipartFormRequestWithMethod:@"POST" URLString:RS_UPLOAD_URL parameters:nil constructingBodyWithBlock:^(id<AFMultipartFormData> formData) {
                NSURL *url = [NSURL URLWithString:params[@"result"]];
                [formData appendPartWithFileURL:url name:@"file" fileName:@"name.mp4" mimeType:@"audio/mpeg" error:nil];
            } error:nil];
        [request setValue:RS_APPKEY forHTTPHeaderField:@"app-key"];
        [request setValue:[CLApplication sharedInstance].token forHTTPHeaderField:@"auth-token"];
        AFURLSessionManager *manager = [[AFURLSessionManager alloc] initWithSessionConfiguration:[NSURLSessionConfiguration defaultSessionConfiguration]];
        NSURLSessionUploadTask *uploadTask = [manager uploadTaskWithStreamedRequest:request progress:nil completionHandler:^(NSURLResponse * _Nonnull response, id  _Nullable responseObject, NSError * _Nullable error) {
                if (error) {
                    NSLog(@"转发错误，不转发1");
                    [self.manager resultWithDetectUrl:params[@"result"] isReTran:NO];
                }
                else {
                    // responseObject为NSDictionary，其他自行处理
                    if ([@"000000" isEqualToString:responseObject[@"code"]]) {
                        NSLog(@"转发成功：转发后视频地址：%@", responseObject[@"data"][@"url"]);
                        [self.manager resultWithDetectUrl:responseObject[@"data"][@"url"] isReTran:YES];
                    }
                    else {
                        NSLog(@"转发错误，不转发2");
                        [self.manager resultWithDetectUrl:params[@"result"] isReTran:NO];
                    }
                }
            }];
        [uploadTask resume];
         */
    }
}

- (void)updateCompleteyWithStatus:(CLStatus)status withDic:(NSDictionary *)params {
    [UIScreen mainScreen].brightness = self.lastBrightness;
    self.videoProcess.hidden = YES;
    self.btn.hidden = YES;
    self.shouldPlay = NO;
    self.actImgView.hidden = YES;
    self.pageView.hidden = YES;
    self.scanImgView.hidden = YES;
    [self.scanImgView.layer removeAllAnimations];
    
    if (status == CLCheckPass) {
        self.msgLabel.text = @"活体检测成功";
        [self.navigationController pushViewController:[[CLSucessViewController alloc] init] animated:YES];
    }
    else if (status == CLOperationTimeout) {
        NSLog(@"-- 超时 --");
        [self.toastView show:CLToastTypeTimeout];
    }
    else if (status == CLCheckNotPass
             || status == CLOnlineUploadFaild
             || status == CLGetConfFaild
             || status == CLOnlineUploadFaild
             || status == CLRecordFaild
             || status == CLSDKError
             || status == CLRPVedioNoMate
             || status == CLRPVedioNoFace
             || status == CLRPVedioMoreFace
             || status == CLRPVideoIllumination
             || status == CLRPVedioAngle
             || status == CLRPVedioBlue
             || status == CLRPVedioMask
             || status == CLRPVedioSmall
             || status == CLNetError
             || status == CLRPVedioError) {
        self.msgLabel.text = @"活体检测失败";
        NSLog(@"%@", [NSString stringWithFormat:@"活体检测失败 {%lu}", (unsigned long)status]);
        [self.scanImgView.layer removeAllAnimations];
        if (status == CLNetError) {
            [self.toastView show:CLToastTypeNetError];
        }
        else {
            [self.toastView show:CLToastTypeFaild];
        }
    }
}

- (void)startLiveDetectWithActionsHandler {
    [self.manager setTimeoutInterval:30];
    [self.manager setActionNumber:1 securityLevel:CLSecurityLevelHigh];
    [self.manager setActionDuration:ActionDuration];
    __weak typeof (self)weakSelf = self;
    [self.manager startLiveDetectWithActionsHandler:^(NSDictionary * _Nonnull params) {
        if (params) {
            dispatch_sync(dispatch_get_main_queue(), ^{
                [weakSelf updateTitleWithDic:params];
            });
        }
    } completionHandler:^(CLStatus status, NSDictionary * _Nullable params) {
        dispatch_sync(dispatch_get_main_queue(), ^{
            [weakSelf updateCompleteyWithStatus:status withDic:params];
        });
    }];
}


#pragma mark - Music

- (void)mucBtnClick:(UIButton *)btn {
    if (btn.selected) {
        self.shouldPlay = YES;
        [btn setImage:[UIImage imageNamed:@"muc_on"] forState:UIControlStateNormal];
    }
    else {
        self.shouldPlay = NO;
        [btn setImage:[UIImage imageNamed:@"muc_off"] forState:UIControlStateNormal];
    }
    btn.selected = !btn.selected;
}

- (void)playActionMusic:(NSString *)musicName {
    NSURL *fileUrl = [[NSBundle mainBundle] URLForResource:musicName withExtension:@"mp3"];
    self.player = [[AVAudioPlayer alloc] initWithContentsOfURL:fileUrl error:nil];
    if (self.player) {
        [self.player prepareToPlay];
    }
    if (self.shouldPlay) {
        [self.player play];
    } else {
        [self.player stop];
    }
}


#pragma mark - dealloc

- (void)dealloc {
    NSLog(@"dealloc");
    [self.toastView hidden];
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)enterBack {
    [self.manager stopLiveDetect];
}

- (void)becomeActive {
    [self.toastView hidden];
    [self retryButtonDidTipped:nil];
}


#pragma mark - CLTimeoutToastViewDelegate

- (void)retryButtonDidTipped:(UIButton * _Nullable)sender {
    [UIScreen mainScreen].brightness = 1.0;
    [self reset];
    [self startLiveDetectWithActionsHandler];
}

- (void)backHomePageButtonDidTipped:(UIButton * _Nullable)sender {
    [self.navigationController popToRootViewControllerAnimated:YES];
}

@end
