//
//  CLOCRManager.h
//  CLOCRSDK
//
//  Created by ydong on 2022/3/29.
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>
#import <CLCore/CLCoreHttpResopnse.h>

NS_ASSUME_NONNULL_BEGIN

typedef void(^CLCoreNetComplete) (CLCoreHttpResopnse * _Nullable httpResopnse);

@interface CLOCRManager : NSObject

/// 设置SDK token，作为后续操作凭证
/// @param token                    从后台获取的token
+ (void)setToken:(NSString *)token;


/// 身份证OCR接口    ⚠️ 先调用人像面（图片大小建议压缩为1M以内，超过会被SDK压缩）
/// @param image                    身份证图片
/// @param isFront                  YES:为人像面 NO:为国徽面
/// @param complete                 结果回调（非主线程） data数据为字典，格式如下：
                                    /*
                                        国徽面数据：  {
                                            "expire_date"       = "失效日期";
                                            "issuing_authority" = "签发机关";
                                            "issuing_date"      = "签发日期";
                                        };
                                        人像面数据： {
                                            "address"    = "地址";
                                            "brith_day"  = "出生日期";
                                            "id_card_no" = "身份证号";
                                            "name"       = "姓名";
                                            "nation"     = "民族";
                                            "sex"        = "性别";
                                        };
                                    */
+ (void)realPersonOCR:(UIImage *)image isFront:(BOOL)isFront complete:(CLCoreNetComplete)complete;


/// 身份证OCR接口    识别人像面和国徽面（单图片大小建议压缩为1M以内，超过会被SDK压缩）
/// @param frontImage               人像面照片
/// @param backImage                国徽面照片
/// @param complete                结果回调（非主线程） data数据为字典，格式如下：
                                    /*    {
                                            back =     {
                                                "expire_date"       = "失效日期";
                                                "issuing_authority" = "签发机关";
                                                "issuing_date"      = "签发日期";
                                            };
                                            front =  {
                                                "address"    = "地址";
                                                "brith_day"  = "出生日期";
                                                "id_card_no" = "身份证号";
                                                "name"       = "姓名";
                                                "nation"     = "民族";
                                                "sex"        = "性别";
                                            };
                                        }
                                    */
+ (void)realPersonOCR:(UIImage *)frontImage backImage:(UIImage *)backImage complete:(CLCoreNetComplete)complete;


/// 身份证认证
/// @param name                     姓名
/// @param idNumber                 身份证号码
/// @param complete                结果回调（非主线程） data数据为字典，格式如下：
                                    /*{
                                        "order_no"  : "业务唯一流水号",
                                        "city"      : "城市",
                                        "country"   : "县区",
                                        "gender"    : "性别：1：男、2：女",
                                        "age"       : "年龄",
                                        "remark"    : "备注，例：一致",
                                        "birthday"  : "生日，格式是yyyyMMdd",
                                        "result"    : "返回结果： 01-认证一致(收费) 02-认证不一致(收费) 03-认证不确定（收费）   04-认证失败(不收费)",
                                        "handle_time" : "查询时间 例：2018-04-09 15:05:01",
                                        "province"  : "省份"
                                   }*/
+ (void)realPersonDectect:(NSString *)name idNumber:(NSString *)idNumber complete:(CLCoreNetComplete)complete;

/// 获取SDK版本号
+ (NSString *)getSDKVersion;

@end

NS_ASSUME_NONNULL_END
