//
//  CLEditImageViewController.m
//  CLRealPersonDemo
//
//  Created by ydong on 2021/6/21.
//  Copyright © 2021 ydong. All rights reserved.
//

#import "CLEditImageViewController.h"
#import "CLRealResultViewController.h"

@interface CLEditImageViewController () {
    CGFloat _height;
    CGFloat _width;
    CGFloat _orignY;
    int _orientation;
}

@property(nonatomic, strong) UIImage *editImage;
@property(nonatomic, strong) UIView *pinchView;
@property(nonatomic, strong) UIImageView *bgImageView;
@property(nonatomic, assign) CGPoint orBgImgViewPoint;
@property(nonatomic, assign) CGRect orBgImgViewRect;
@property(nonatomic, assign) float imgScale;
@property(nonatomic, strong) UIImageView *rxImageView;
@property(nonatomic, strong) UIView *boxImageView;

@end

@implementation CLEditImageViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.view.backgroundColor = UIColor.whiteColor;
    self.editImage = self.image;
    self.imgScale = 1.0;
    
    self.bgImageView = [[UIImageView alloc] initWithFrame:self.view.frame];
    self.bgImageView.image = self.editImage;
    self.bgImageView.contentMode = UIViewContentModeScaleAspectFit;
    [self.view addSubview:self.bgImageView];
    
    _height = floorf((SCREEN_WIDTH-KWidthScale*20)*54/85.6);
    _orignY = ceil((self.view.frame.size.height-_height)/2);
    _width = floorf(SCREEN_WIDTH-KWidthScale*20);
    CGRect smallRect = CGRectMake(KWidthScale*10, _orignY, _width, _height);

    UIView *imgView = [[UIView alloc] initWithFrame:smallRect];
    imgView.layer.borderColor = UIColor.whiteColor.CGColor;
    imgView.layer.borderWidth = 1;
    imgView.layer.cornerRadius = 5;
    [self.view addSubview:imgView];
    self.boxImageView = imgView;
    
    UIImageView *imageView = [[UIImageView alloc] init];
    if (self.emblem) {
        imageView.image = [UIImage imageNamed:@"gh"];
        imageView.frame = CGRectMake(26*KWidthScale, _orignY+15*KHeightScale, 63*KWidthScale, 68*KWidthScale);
    }
    else {
        imageView.image = [UIImage imageNamed:@"rx"];
        imageView.frame = CGRectMake(SCREEN_WIDTH-KWidthScale*(23+114), _orignY+36*KHeightScale, 114*KWidthScale, 119*KWidthScale);
    }
    [self.view addSubview:imageView];
    self.rxImageView = imageView;
    
    self.pinchView = [[UIView alloc] initWithFrame:self.view.bounds];
    self.pinchView.backgroundColor = [UIColor colorWithWhite:0 alpha:.5];
    UIBezierPath *path = [UIBezierPath bezierPathWithRect:self.view.bounds];
    [path appendPath:[[UIBezierPath bezierPathWithRoundedRect:smallRect cornerRadius:5] bezierPathByReversingPath]];
    CAShapeLayer *shapeLayer = [CAShapeLayer layer];
    shapeLayer.path = path.CGPath;
    [self.pinchView.layer setMask:shapeLayer];
    [self.view addSubview:self.pinchView];
  
    UILabel *label = [[UILabel alloc] init];
    label.text = @"适当调整方向，将图片置于虚线框内";
    label.font = [UIFont systemFontOfSize:14];
    label.textAlignment = NSTextAlignmentCenter;
    label.textColor = UIColor.whiteColor;
    label.backgroundColor = HexColor(@"#515459");
    label.layer.cornerRadius = 2;
    [self.view addSubview:label];
    [label mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(self.view.mas_top).offset(80*KHeightScale);
        make.left.mas_equalTo(self.view.mas_left).offset(50);
        make.right.mas_equalTo(self.view.mas_right).offset(-50);
        make.height.mas_equalTo(36);
    }];
    
    UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
    [button setTitle:@"取消" forState:UIControlStateNormal];
    [self.view addSubview:button];
    [button addTarget:self action:@selector(cancleClick) forControlEvents:UIControlEventTouchUpInside];
    [button mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(self.view.mas_bottom).offset(-100*KHeightScale);
        make.left.mas_equalTo(self.view.mas_left).offset(25*KWidthScale);
        make.size.mas_equalTo(CGSizeMake(45, 45));
    }];
    
    UIButton *leftBtn = [UIButton buttonWithType:UIButtonTypeCustom];
    [leftBtn setImage:[UIImage  imageNamed:@"left"] forState:UIControlStateNormal];
    leftBtn.imageEdgeInsets = UIEdgeInsetsMake(7.5, 7.5, 7.5, 7.5);
    [self.view addSubview:leftBtn];
    [leftBtn addTarget:self action:@selector(leftClick) forControlEvents:UIControlEventTouchUpInside];
    [leftBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(self.view.mas_bottom).offset(-100*KHeightScale);
        make.centerX.mas_equalTo(self.view.mas_centerX).offset(-50*KWidthScale);
        make.size.mas_equalTo(CGSizeMake(45, 45));
    }];
    
    UIButton *rightBtn = [UIButton buttonWithType:UIButtonTypeCustom];
    [rightBtn setImage:[UIImage  imageNamed:@"right"] forState:UIControlStateNormal];
    rightBtn.imageEdgeInsets = UIEdgeInsetsMake(7.5, 7.5, 7.5, 7.5);
    [self.view addSubview:rightBtn];
    [rightBtn addTarget:self action:@selector(rightClick) forControlEvents:UIControlEventTouchUpInside];
    [rightBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(self.view.mas_bottom).offset(-100*KHeightScale);
        make.centerX.mas_equalTo(self.view.mas_centerX).offset(50*KWidthScale);
        make.size.mas_equalTo(CGSizeMake(45, 45));
    }];
    
    UIButton *entBtn = [UIButton buttonWithType:UIButtonTypeCustom];
    [entBtn setTitle:@"确定" forState:UIControlStateNormal];
    entBtn.backgroundColor = HexColor(@"#3F75FC");
    entBtn.layer.cornerRadius = 2;
    [self.view addSubview:entBtn];
    [entBtn addTarget:self action:@selector(entClick) forControlEvents:UIControlEventTouchUpInside];
    [entBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(self.view.mas_bottom).offset(-100*KHeightScale+4.5);
        make.right.mas_equalTo(self.view.mas_right).offset(-25*KWidthScale);
        make.size.mas_equalTo(CGSizeMake(60, 36));
    }];
    
    UIPinchGestureRecognizer *pinch = [[UIPinchGestureRecognizer alloc] initWithTarget:self action:@selector(pinch:)];
    [self.pinchView addGestureRecognizer:pinch];
    UIPanGestureRecognizer *pan = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(pan:)];
    pan.maximumNumberOfTouches = 1;
    [self.pinchView addGestureRecognizer:pan];
    [self reSize];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    self.rxImageView.hidden = NO;
    self.boxImageView.hidden = NO;
    self.navigationController.navigationBarHidden = YES;
}

-  (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    self.navigationController.navigationBarHidden = NO;
}

- (void)pinch:(UIPinchGestureRecognizer *)pinch {
    if (pinch.state == UIGestureRecognizerStateBegan) {
        self.orBgImgViewRect = self.bgImageView.frame;
    }
    else if ( pinch.state == UIGestureRecognizerStateChanged) {
        self.imgScale = pinch.scale;
        CGFloat width = self.orBgImgViewRect.size.width*self.imgScale;
        CGFloat height = self.orBgImgViewRect.size.height*self.imgScale;
        
        self.bgImageView.frame = CGRectMake(self.orBgImgViewRect.origin.x-(width - self.orBgImgViewRect.size.width)/2,
                                            self.orBgImgViewRect.origin.y-(height - self.orBgImgViewRect.size.height)/2,
                                            width, height);
    }
}

- (void)pan:(UIPanGestureRecognizer *)pan {
    if (pan.state == UIGestureRecognizerStateBegan ) {
        self.orBgImgViewPoint = self.bgImageView.frame.origin;
    }
    else if (pan.state == UIGestureRecognizerStateChanged) {
        CGPoint point = [pan translationInView:pan.view.superview];
        self.bgImageView.frame = CGRectMake(self.orBgImgViewPoint.x + point.x, self.orBgImgViewPoint.y +point.y, self.bgImageView.frame.size.width, self.bgImageView.frame.size.height);
    }
}

- (void)reSize {
    CGFloat viewHeight = self.view.frame.size.height;
    CGFloat viewWidth = self.view.frame.size.width;
    CGFloat imgViewHeight = _height;
    CGFloat imgViewWidth = _width;
    if (self.editImage.size.width/_width > self.editImage.size.height/_height) {
        imgViewWidth = (_height/self.editImage.size.height)*self.editImage.size.width;
    }
    else {
        imgViewHeight = (_width/self.editImage.size.width)*self.editImage.size.height;
    }
    self.bgImageView.frame = CGRectMake((viewWidth-imgViewWidth)/2, (viewHeight-imgViewHeight)/2, imgViewWidth, imgViewHeight);
    self.bgImageView.image = self.editImage;
}

- (void)cancleClick {
    self.bgImageView.hidden = YES;
    self.boxImageView.hidden = YES;
    [self.navigationController popViewControllerAnimated:YES];
}

- (void)leftClick {
    [UIView animateWithDuration:0.15 animations:^{
        self.bgImageView.transform = CGAffineTransformRotate(self.bgImageView.transform,-M_PI/2);
    }];
}

- (void)rightClick {
    [UIView animateWithDuration:0.15 animations:^{
        self.bgImageView.transform = CGAffineTransformRotate(self.bgImageView.transform,M_PI/2);
    }];
}

- (void)entClick {
    self.rxImageView.hidden = YES;
    self.boxImageView.hidden = YES;
    
    CGRect screenRect = [UIScreen mainScreen].bounds;
    UIGraphicsBeginImageContextWithOptions(screenRect.size, 0, 0);
    [self.view.layer renderInContext:UIGraphicsGetCurrentContext()];
    UIImage *img = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    CGImageRef newImageRef = CGImageCreateWithImageInRect(img.CGImage,  CGRectMake(ceil(KWidthScale*10)*img.scale, _orignY*img.scale, _width*img.scale, _height*img.scale));
    UIImage *newImage = [UIImage imageWithCGImage:newImageRef];
    CLRealResultViewController *vc = [CLRealResultViewController new];
    vc.image = newImage;
    vc.emblem = self.emblem;
    if (self.ocrAll && self.imageBlock) {
        [self.navigationController popViewControllerAnimated:YES];
        self.imageBlock(newImage);
        return;
    }
    [self.navigationController pushViewController:vc animated:YES];
}

@end
