//
//  CLOCRViewController.m
//  CLRealPersonDemo
//
//  Created by ydong on 2021/7/29.
//  Copyright © 2021 ydong. All rights reserved.
//

#import "CLOCRViewController.h"
#import "CLRealViewController.h"
#import "CLRealEditResultViewController.h"

@interface CLOCRViewController ()

@property (nonatomic, strong)UIImage *frontImage;
@property (nonatomic, strong)UIImage *backImage;
@property (weak, nonatomic) IBOutlet UIButton *nextBtn;
@property (weak, nonatomic) IBOutlet UIButton *frontBtn;
@property (weak, nonatomic) IBOutlet UIButton *backBtn;

@end

@implementation CLOCRViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.title = @"上传身份证";
    self.view.backgroundColor = HexColor(@"FFFFFF");
    self.navigationController.navigationBar.tintColor = UIColor.blackColor;
    self.navigationController.navigationBar.translucent = NO;
    [self.navigationController setNavigationBarHidden:NO animated:YES];
    self.nextBtn.layer.cornerRadius = 5;
    self.frontBtn.layer.cornerRadius = 5;
    self.frontBtn.layer.masksToBounds = YES;
    self.backBtn.layer.cornerRadius = 5;
    self.backBtn.layer.masksToBounds = YES;
}

- (IBAction)frontClick:(UIButton *)sender {
    CLRealViewController *vc = [[CLRealViewController alloc] init];
    vc.ocrAll = YES;
    vc.imageBlock = ^(UIImage *image){
        [sender setImage:image forState:UIControlStateNormal];
        self.frontImage = image;
    };
    [self.navigationController pushViewController:vc animated:YES];
}

- (IBAction)backClick:(UIButton *)sender {
    CLRealViewController *vc = [[CLRealViewController alloc] init];
    vc.emblem = YES;
    vc.ocrAll = YES;
    vc.imageBlock = ^(UIImage *image){
        [sender setImage:image forState:UIControlStateNormal];
        self.backImage = image;
    };
    [self.navigationController pushViewController:vc animated:YES];
}

- (IBAction)ocrBtnClick:(UIButton *)sender {
    UIActivityIndicatorView *indicatorView = [[UIActivityIndicatorView alloc]initWithFrame:self.view.frame];
    indicatorView.color = [UIColor blueColor];
    indicatorView.activityIndicatorViewStyle = UIActivityIndicatorViewStyleWhiteLarge;
    [indicatorView startAnimating];
    [self.view addSubview:indicatorView];
    if (!self.frontImage || !self.backImage) {
        [indicatorView stopAnimating];
        [indicatorView removeFromSuperview];
        [CLToastView showNotice:@"图片参数为空"];
        return;
    }
    [CLOCRManager realPersonOCR:self.frontImage backImage:self.backImage complete:^(CLCoreHttpResopnse * _Nullable httpResopnse) {
        NSLog(@"%@", httpResopnse.data);
        dispatch_async(dispatch_get_main_queue(), ^{
            [indicatorView stopAnimating];
            [indicatorView removeFromSuperview];
            if ([httpResopnse.data isKindOfClass:NSDictionary.class] && [@"000000" isEqualToString: httpResopnse.code]) {
                NSString *errStr = @"";
                if([[httpResopnse.data[@"front"] allKeys] containsObject:@"msg"]) {
                    errStr = [NSString stringWithFormat:@"人像面:%@" ,httpResopnse.data[@"front"][@"msg"]];
                }
                 if ([[httpResopnse.data[@"back"] allKeys] containsObject:@"msg"]) {
                     errStr = [NSString stringWithFormat:@"%@  国徽面:%@", errStr,httpResopnse.data[@"back"][@"msg"]];
                }
                if (errStr.length>0) {
                    [CLToastView showNotice:errStr];
                    return;
                }
                else {
                    if ( [[httpResopnse.data[@"front"] allKeys] containsObject:@"name"]) {
                        [CLApplication sharedInstance].name = httpResopnse.data[@"front"][@"name"];
                        [CLApplication sharedInstance].idNumber = httpResopnse.data[@"front"][@"id_card_no"];
                        [CLApplication sharedInstance].img = self.frontImage;
                        CLRealEditResultViewController *editVc = [[CLRealEditResultViewController alloc] init];
                        [self.navigationController pushViewController:editVc animated:YES];
                    }
                    else {
                        [CLToastView showNotice:@"没有识别到名字"];
                    }
                }
            }
            else {
                if (httpResopnse.code) {
                    [CLToastView showNotice:[NSString stringWithFormat:@"识别出错:%@" ,httpResopnse.message]];
                }
                else {
                    [CLToastView showNotice:@"识别出错:检查网络"];
                }
            }
        });
    }];
}

- (IBAction)otherClick:(id)sender {
    CLRealViewController *vc = [[CLRealViewController alloc] init];
    [self.navigationController pushViewController:vc animated:YES];
}

@end
