//
//  CLRealViewController.m
//  CLRealPersonDemo
//
//  Created by ydong on 2021/6/17.
//  Copyright © 2021 ydong. All rights reserved.
//

#import "CLRealViewController.h"
#import <AVFoundation/AVFoundation.h>
#import "CLRealResultViewController.h"
#import "TZImagePickerController.h"
#import "CLEditImageViewController.h"

@interface CLRealViewController ()<AVCaptureMetadataOutputObjectsDelegate,UIAlertViewDelegate, TZImagePickerControllerDelegate>

@property(nonatomic, strong) AVCaptureDevice *device;

@property(nonatomic, strong) AVCaptureDeviceInput *input;

@property(nonatomic, strong) AVCaptureMetadataOutput *output;

@property(nonatomic, strong) AVCaptureStillImageOutput *ImageOutPut;

@property(nonatomic, strong) AVCaptureSession *session;

@property(nonatomic, strong) AVCaptureVideoPreviewLayer *previewLayer;

@property(nonatomic, strong) UIView *bgView;

@end

@implementation CLRealViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.view.backgroundColor = HexColor(@"FFFFFF");
    self.navigationController.navigationBar.tintColor = UIColor.blackColor;
    self.navigationController.navigationBar.translucent = NO;
    [self.navigationController setNavigationBarHidden:NO animated:YES];
    if (self.emblem) {
        self.title = @"身份证国徽面拍摄";
    }
    else {
        self.title = @"身份证人像面拍摄";
    }
    
    if ([self canUserCamear]) {
        [self customCamera];
        [self initBottomUI];
    }
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    __weak typeof (self)weakSelf =  self;
    dispatch_async(dispatch_get_global_queue(0, 0), ^{
        [weakSelf.session startRunning];
    });
    if ([_device lockForConfiguration:nil]) {
        if ([_device isWhiteBalanceModeSupported:AVCaptureWhiteBalanceModeAutoWhiteBalance]) {
            [_device setWhiteBalanceMode:AVCaptureWhiteBalanceModeAutoWhiteBalance];
        }
        [_device unlockForConfiguration];
    }
}

- (void)customCamera{
    self.device = [AVCaptureDevice defaultDeviceWithMediaType:AVMediaTypeVideo];
    self.input = [[AVCaptureDeviceInput alloc] initWithDevice:self.device error:nil];
    self.output = [[AVCaptureMetadataOutput alloc] init];
    self.ImageOutPut = [[AVCaptureStillImageOutput alloc] init];
    self.session = [[AVCaptureSession alloc] init];
    if ([self.session canSetSessionPreset:AVCaptureSessionPreset1280x720]) {
        self.session.sessionPreset = AVCaptureSessionPreset1280x720;
    }
    if ([self.session canAddInput:self.input]) {
        [self.session addInput:self.input];
    }
    if ([self.session canAddOutput:self.ImageOutPut]) {
        [self.session addOutput:self.ImageOutPut];
    }
    self.previewLayer = [[AVCaptureVideoPreviewLayer alloc] initWithSession:self.session];
    CGFloat height = (SCREEN_WIDTH-KWidthScale*65)*54/85.6;
    self.previewLayer.frame = CGRectMake(KWidthScale*32, KHeightScale*39, SCREEN_WIDTH-KWidthScale*65, height);
    self.previewLayer.videoGravity = AVLayerVideoGravityResizeAspectFill;
    self.previewLayer.cornerRadius = 5;
    self.bgView.frame = CGRectMake(0, 0, SCREEN_WIDTH, KHeightScale*(39+67)+height);
    [self.bgView.layer addSublayer:self.previewLayer];
    
    UIImageView *imageView = [[UIImageView alloc] init];
    if (self.emblem) {
        imageView.image = [UIImage imageNamed:@"gh"];
        imageView.frame = CGRectMake(46*KWidthScale, KHeightScale*39+ 15*KWidthScale, 63*KWidthScale, 68*KWidthScale);
    }
    else {
        imageView.image = [UIImage imageNamed:@"rx"];
        imageView.frame = CGRectMake(SCREEN_WIDTH-KWidthScale*(12+32+114), KHeightScale*39+ 33*KWidthScale, 114*KWidthScale, 119*KWidthScale);
    }
    [self.bgView addSubview:imageView];
    
    UILabel *label = [[UILabel alloc] init];
    label.numberOfLines = 0;
    label.textAlignment = NSTextAlignmentCenter;
    label.font = [UIFont systemFontOfSize:14];
    label.textColor = HexColor(@"606581");
    if (self.emblem) {
        label.text = @"将身份证（背面）置于此区域，避免晃动和反光";
    }
    else {
        label.text = @"将身份证人像页置于此区域，避免晃动和反光";
    }
    [self.bgView addSubview:label];
    [label mas_makeConstraints:^(MASConstraintMaker *make) {
        make.bottom.mas_equalTo(self.bgView.mas_bottom);
        make.height.mas_equalTo(KHeightScale*67);
        make.left.mas_equalTo(KWidthScale*32);
        make.right.mas_equalTo(-KWidthScale*32);
    }];
}

- (UIView *)bgView {
    if (!_bgView) {
        _bgView = [[UIView alloc] init];
        _bgView.backgroundColor = HexColor(@"FAFAFA");
        [self.view addSubview:_bgView];
    }
    return _bgView;
}

- (void)initBottomUI {
    UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
    [button setImage:[UIImage  imageNamed:@"tphoto"] forState:UIControlStateNormal];
    [self.view addSubview:button];
    [button addTarget:self action:@selector(shutterCamera) forControlEvents:UIControlEventTouchUpInside];
    [button mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(self.bgView.mas_bottom).offset(37*KHeightScale);
        make.centerX.mas_equalTo(self.view.mas_centerX);
        make.size.mas_equalTo(CGSizeMake(73, 73));
    }];
    
    UIButton *lButton = [UIButton buttonWithType:UIButtonTypeCustom];
    [lButton setImage:[UIImage imageNamed:@"photo"] forState:UIControlStateNormal];
    [lButton setImage:[UIImage imageNamed:@"photo"] forState:UIControlStateHighlighted];
    [lButton setTitle:@"相册" forState:UIControlStateNormal];
    lButton.titleLabel.textColor = HexColor(@"#747991");
    [lButton setTitleColor:HexColor(@"#747991") forState:UIControlStateNormal];
    [lButton addTarget:self action:@selector(checkLocalPhoto) forControlEvents:UIControlEventTouchUpInside];
    lButton.titleLabel.font = [UIFont systemFontOfSize:12];
    lButton.imageEdgeInsets = UIEdgeInsetsMake(0, 12, 24, 12);
    lButton.titleEdgeInsets = UIEdgeInsetsMake(40, -27, 0, 0);
    [self.view addSubview:lButton];
    [lButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.bottom.mas_equalTo(button.mas_bottom);
        make.right.mas_equalTo(button.mas_left).offset(-47*KWidthScale);
        make.size.mas_equalTo(CGSizeMake(54, 54));
    }];
    
    UIButton *rButton = [UIButton buttonWithType:UIButtonTypeCustom];
    [rButton setImage:[UIImage imageNamed:@"light"] forState:UIControlStateNormal];
    [rButton setImage:[UIImage imageNamed:@"light"] forState:UIControlStateHighlighted];
    [rButton setTitle:@"手电筒" forState:UIControlStateNormal];
    [rButton setTitleColor:HexColor(@"#747991") forState:UIControlStateNormal];
    rButton.titleLabel.font = [UIFont systemFontOfSize:12];
    rButton.imageEdgeInsets = UIEdgeInsetsMake(0, 12, 24, 12);
    rButton.titleEdgeInsets = UIEdgeInsetsMake(40, -27, 0, 0);
    [rButton addTarget:self action:@selector(flashOn:) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:rButton];
    [rButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.bottom.mas_equalTo(button.mas_bottom);
        make.left.mas_equalTo(button.mas_right).offset(47*KWidthScale);
        make.size.mas_equalTo(CGSizeMake(54, 54));
    }];
    
    UIView *lView = [[UIView alloc] init];
    lView.backgroundColor  = HexColor(@"#D9D9D9");
    [self.view addSubview:lView];
    [lView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(button.mas_bottom).offset(56*KHeightScale);
        make.left.mas_equalTo(self.view.mas_left).offset(30*KWidthScale);
        make.size.mas_equalTo(CGSizeMake(120*KWidthScale, 1));
    }];
    UIView *rView = [[UIView alloc] init];
    rView.backgroundColor  = HexColor(@"#D9D9D9");
    [self.view addSubview:rView];
    [rView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(button.mas_bottom).offset(56*KHeightScale);
        make.right.mas_equalTo(self.view.mas_right).offset(-30*KWidthScale);
        make.size.mas_equalTo(CGSizeMake(120*KWidthScale, 1));
    }];
    
    UILabel *label = [[UILabel alloc] init];
    label.numberOfLines = 0;
    label.textAlignment = NSTextAlignmentCenter;
    label.font = [UIFont systemFontOfSize:13];
    label.textColor = HexColor(@"#656477");
    label.text = @"拍摄标准";
    [self.view addSubview:label];
    [label mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(button.mas_bottom).offset(50*KHeightScale);
        make.centerX.mas_equalTo(self.view.mas_centerX);
        make.size.mas_equalTo(CGSizeMake(55, 13));
    }];
    
    int i = 0;
    for (NSString *str in @[@"标准拍摄", @"边框缺失", @"照片模糊", @"闪光强烈"]) {
        UIImageView *imageView = [[UIImageView alloc] init];
        [self.view addSubview:imageView];
        [imageView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.mas_equalTo(label.mas_bottom).offset(14*KHeightScale);
            make.left.mas_equalTo(self.view.mas_left).offset(30*KWidthScale+(73+7.5)*KWidthScale*i);
            make.size.mas_equalTo(CGSizeMake(73*KWidthScale, 48*KHeightScale));
        }];
        i++;
        imageView.image = [UIImage imageNamed:[NSString stringWithFormat:@"img_%d", i]];
        
        UIImageView *iconImgView = [[UIImageView alloc] init];
        if (i==1) {
            iconImgView.image = [UIImage imageNamed:@"gou"];
        }
        else {
            iconImgView.image = [UIImage imageNamed:@"cha"];
        }
        [self.view addSubview:iconImgView];
        [iconImgView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.mas_equalTo(imageView.mas_bottom).offset(15*KHeightScale);
            make.left.mas_equalTo(imageView.mas_left);
            make.size.mas_equalTo(CGSizeMake(10*KWidthScale, 7*KHeightScale));
        }];
        
        UILabel *labelB = [[UILabel alloc] init];
        labelB.font = [UIFont systemFontOfSize:12];
        labelB.textColor = HexColor(@"#656477");
        labelB.text = str;
        [self.bgView addSubview:labelB];
        [labelB mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.mas_equalTo(imageView.mas_bottom).offset(12*KHeightScale);
            make.left.mas_equalTo(iconImgView.mas_right).offset(4*KWidthScale);
            make.size.mas_equalTo(CGSizeMake(59*KWidthScale, 12*KHeightScale));
        }];
    }
}

- (void)shutterCamera {
    AVCaptureConnection * videoConnection = [self.ImageOutPut connectionWithMediaType:AVMediaTypeVideo];
    if (!videoConnection) {
        NSLog(@"take photo failed!");
        return;
    }
    [self.ImageOutPut captureStillImageAsynchronouslyFromConnection:videoConnection completionHandler:^(CMSampleBufferRef imageDataSampleBuffer, NSError *error) {
        if (imageDataSampleBuffer == NULL) {
            return;
        }
        NSData *imageData = [AVCaptureStillImageOutput jpegStillImageNSDataRepresentation:imageDataSampleBuffer];
        [self.session stopRunning];
        UIImage *image = [UIImage imageWithData:imageData];
        CGFloat height = image.size.height;
        CGFloat rHeight = image.size.width*54/85.6;
        CGRect r = CGRectMake(floorf((height-rHeight)/2), 0, floorf(rHeight), image.size.width);
        CGImageRef imageRef = CGImageCreateWithImageInRect(image.CGImage, r);
        UIImage *thumbScale = [UIImage imageWithCGImage:imageRef scale:0 orientation:UIImageOrientationRight];
        CGImageRelease(imageRef);
        CLRealResultViewController *vc = [CLRealResultViewController new];
        vc.image = thumbScale;
        vc.emblem = self.emblem;
        if (self.ocrAll && self.imageBlock) {
            self.imageBlock(thumbScale);
            [self.navigationController popViewControllerAnimated:YES];
            return;
        }
        [self.navigationController pushViewController:vc animated:YES];
    }];
}

- (void)flashOn:(UIButton *)button {
    if ([_device lockForConfiguration:nil]) {
        if (button.selected) {
            if ([_device isTorchModeSupported:AVCaptureTorchModeOff]) {
                [_device setTorchMode:AVCaptureTorchModeOff];
            }
        }else{
            if ([_device isTorchModeSupported:AVCaptureTorchModeOn]) {
                [_device setTorchMode:AVCaptureTorchModeOn];
            }
        }
        [_device unlockForConfiguration];
    }
    button.selected = !button.selected;
}

- (void)checkLocalPhoto{
    TZImagePickerController *imagePicker = [[TZImagePickerController alloc] initWithMaxImagesCount:1 delegate:self];
    imagePicker.navigationBar.barTintColor = [UIColor whiteColor];
    imagePicker.navigationBar.tintColor = UIColor.whiteColor;
    imagePicker.naviTitleColor = UIColor.blackColor;
    imagePicker.barItemTextColor = UIColor.blackColor;
    imagePicker.minPhotoWidthSelectable = 720;
    imagePicker.allowPreview = NO;
    imagePicker.allowPickingVideo = NO;
    imagePicker.statusBarStyle = UIStatusBarStyleDefault;
    imagePicker.modalPresentationStyle = UIModalPresentationFullScreen;
    [self presentViewController:imagePicker animated:YES completion:nil];
}

- (void)imagePickerController:(TZImagePickerController *)picker didFinishPickingPhotos:(NSArray<UIImage *> *)photos sourceAssets:(NSArray *)assets isSelectOriginalPhoto:(BOOL)isSelectOriginalPhoto{
    if (photos && photos.count>0) {
        CLEditImageViewController *vc = [CLEditImageViewController new];
        vc.image = photos[0];
        vc.emblem = self.emblem;
        vc.ocrAll = self.ocrAll;
        __weak typeof (self)weakSelf = self;
        vc.imageBlock = ^(UIImage *img){
            if (weakSelf.ocrAll && weakSelf.imageBlock) {
                weakSelf.imageBlock(img);
                [weakSelf.navigationController popViewControllerAnimated:YES];
                return;
            }
        };
        [self.navigationController pushViewController:vc animated:YES];
    }
}

- (BOOL)canUserCamear{
    AVAuthorizationStatus authStatus = [AVCaptureDevice authorizationStatusForMediaType:AVMediaTypeVideo];
    if (authStatus == AVAuthorizationStatusDenied) {
        [CLToastView showNotice:@"没有开启相机权限"];
        return NO;
    }
    else{
        return YES;
    }
    return YES;
}

@end
